/******************************************************************************

   PROJECT:       CVDRIK1
   FILE:          CVDRIKIplImagePyramid.cpp
   AUTHOR:        PRITHWIJIT GUHA
   DATE:
   DESCRIPTION:   Functions for the IplImage Pyramid

******************************************************************************/


/*************************** OTHER INCLUDE FILES *****************************/

#include "CVDRIKMultiChannelImage.h"

#define EPSILON 0.0001

/*****************************************************************************/


/******************************************************************************

                                FUNCTIONS

******************************************************************************/

// Function for Initializing the IplImage Pyramid
// Inputs :-
//			(CVDRIKIplImagePyramid*) iplImgPyr : Pointer to the IplImage Pyramid
// Outputs :-
//			None
// Invoked As : initIplImagePyramid( &iplImgPyr );
void initIplImagePyramid( CVDRIKIplImagePyramid* iplImgPyr )
{
	// Memory Allocation Flag
	iplImgPyr->m_IplImagePyramidMemoryAllocated = false;
	
	// The Image Width at the Highest Resolution
	iplImgPyr->m_HighestResWidth = -1;
	
	// The Image Height at the Highest Resolution
	iplImgPyr->m_HighestResHeight = -1;
	
	// The Number of Channels
	iplImgPyr->m_ChannelNum = -1;
	
	// The Pyramid Size
	iplImgPyr->m_PyramidSize = -1;
	
	// The Pyramid Component Image Array
	iplImgPyr->m_IplImagePyramid = NULL;
	
	// The Scaling Down Factor Along the Width
	iplImgPyr->m_ScaleFactorWidth = -1.0;
	
	// The Scaling Down Factor Along the Height
	iplImgPyr->m_ScaleFactorHeight = -1.0;
}

// Function for Allocating a IplImage Pyramid
// Inputs :-
//			(CVDRIKIplImagePyramid*) iplImgPyr : Pointer to the IplImage Pyramid
//			(int) highestResWidth : Width of the Highest Resolution Image
//			(int) highestResHeight : Height of the Highest Resolution Image
//			(int) channelNum : The Number of Channels of the Pyramid Component Images
//			(float) scaleFactorWidth : The Down Scaling Factor Along the Width
//			(float) scaleFactorHeight : The Down Scaling Factor Along the Height
//			(int) pyramidSize : The Number of Images in the Pyramid
// Outputs :-
//			(unsigned char) errCode : The Error Code of Execution
// Invoked As : errCode = allocateIplImagePyramid( iplImgPyr , highestResWidth , highestResHeight , channelNum , scaleFactorWidth , scaleFactorHeight , pyramidSize );
unsigned char allocateIplImagePyramid( CVDRIKIplImagePyramid* iplImgPyr , int highestResWidth , int highestResHeight , int channelNum , float scaleFactorWidth , float scaleFactorHeight , int pyramidSize )
{
	// Check Inputs
	if( ( iplImgPyr->m_IplImagePyramidMemoryAllocated ) || ( highestResWidth <= 0 ) || ( highestResHeight <= 0 ) || ( channelNum <= 0 ) || ( channelNum > 4 ) || ( scaleFactorWidth < 1 ) || ( scaleFactorHeight < 1 ) || ( pyramidSize <= 0 ) )
	{
		// Error Code 1 : Invalid Input
		return( 1 );
	}
	
	// Check the Validity of the Sizes
	int finalScaleWidth = cvRound( pow( scaleFactorWidth , ( pyramidSize - 1 ) ) );
	int finalScaleHeight = cvRound( pow( scaleFactorHeight , ( pyramidSize - 1 ) ) );
	if( ( highestResWidth < finalScaleWidth ) || ( highestResHeight < finalScaleHeight ) )
	{
		// Error Code 6 : Index Exceeds Array Bound
		return( 6 );
	}
	
	// The Image Width at the Highest Resolution
	iplImgPyr->m_HighestResWidth = highestResWidth;
	
	// The Image Height at the Highest Resolution
	iplImgPyr->m_HighestResHeight = highestResHeight;
	
	// The Number of Channels
	iplImgPyr->m_ChannelNum = channelNum;
	
	// The Pyramid Size
	iplImgPyr->m_PyramidSize = pyramidSize;
	
	// Allocating the Image Pyramid
	iplImgPyr->m_IplImagePyramid = (IplImage**) malloc( pyramidSize * sizeof( IplImage* ) );
	
	// Allocating the Indvidual Images
	float currWidth = scaleFactorWidth * highestResWidth;
	float currHeight = scaleFactorHeight * highestResHeight;
	for( int scaleCount = 0 ; scaleCount < pyramidSize ; ++scaleCount )
	{
		// Update the Current Sizes
		currWidth = currWidth / scaleFactorWidth;
		currHeight = currHeight / scaleFactorHeight;
		
		// Allocate the Image at the Current Resolution
		int width = cvRound( currWidth );
		int height = cvRound( currHeight );
		iplImgPyr->m_IplImagePyramid[ scaleCount ] = 0x00;
		iplImgPyr->m_IplImagePyramid[ scaleCount ] = cvCreateImage( cvSize( width , height ) , IPL_DEPTH_8U , channelNum );
		
		// Check Allocation Status
		if( iplImgPyr->m_IplImagePyramid[ scaleCount ] == 0x00 )
		{
			// Error Code 8 : Memory Allocation Failed
			return( 8 );
		}
	}
	
	// The Scaling Down Factor Along the Width
	iplImgPyr->m_ScaleFactorWidth = scaleFactorWidth;
	
	// The Scaling Down Factor Along the Height
	iplImgPyr->m_ScaleFactorHeight = scaleFactorHeight;
	
	// Memory Allocation Flag
	iplImgPyr->m_IplImagePyramidMemoryAllocated = true;
	
	// Error Code 0 : All Well, That Ends Well
	return( 0 );
}

// Function for Destroying a IplImage Pyramid
// Inputs :-
//			(CVDRIKIplImagePyramid*) iplImgPyr : Pointer to the IplImage Pyramid
// Outputs :-
//			(unsigned char) errCode : The Error Code of Execution
// Invoked As : errCode = destroyIplImagePyramid( iplImgPyr );
unsigned char destroyIplImagePyramid( CVDRIKIplImagePyramid* iplImgPyr )
{
	// Check Inputs
	if( !( iplImgPyr->m_IplImagePyramidMemoryAllocated ) )
	{
		// Error Code 1 : Invalid Input
		return( 1 );
	}
	
	// Memory Allocation Flag
	iplImgPyr->m_IplImagePyramidMemoryAllocated = false;
	
	// The Image Width at the Highest Resolution
	iplImgPyr->m_HighestResWidth = -1;
	
	// The Image Height at the Highest Resolution
	iplImgPyr->m_HighestResHeight = -1;
	
	// The Number of Channels
	iplImgPyr->m_ChannelNum = -1;
	
	// Destroying the Pyramid Component Images
	for( int scaleCount = 0 ; scaleCount < iplImgPyr->m_PyramidSize ; ++scaleCount )
	{
		// Release the Image
		cvReleaseImage( &iplImgPyr->m_IplImagePyramid[ scaleCount ] );
		iplImgPyr->m_IplImagePyramid[ scaleCount ] = 0x00;
	}
	free( iplImgPyr->m_IplImagePyramid );
	iplImgPyr->m_IplImagePyramid = NULL;
	
	// The Pyramid Size
	iplImgPyr->m_PyramidSize = -1;
	
	// The Scaling Down Factor Along the Width
	iplImgPyr->m_ScaleFactorWidth = -1.0;
	
	// The Scaling Down Factor Along the Height
	iplImgPyr->m_ScaleFactorHeight = -1.0;
	
	// Error Code 0 : All Well, That Ends Well
	return( 0 );
}

// Function for Setting the Image Data of the IplImage Pyramid
// Inputs :-
//			(CVDRIKIplImagePyramid*) iplImgPyr : Pointer to the IplImage Pyramid
//			(IplImage*) img : Pointer to the Image at the Highest Resolution
// Outputs :-
//			(unsigned char) errCode : The Error Code of Execution
// Invoked As : errCode = setIplImagePyramidData( &iplImgPyr , &img );
unsigned char setIplImagePyramidData( CVDRIKIplImagePyramid* iplImgPyr , IplImage* img )
{
	// Check Inputs
	if( ( !( iplImgPyr->m_IplImagePyramidMemoryAllocated ) ) || ( img->imageSize <= 0 ) )
	{
		// Error Code 1 : Invalid Inputs
		return( 1 );
	}
	
	// Check Data Size Match
	if( ( iplImgPyr->m_HighestResWidth != img->width ) || ( iplImgPyr->m_HighestResHeight != img->height ) || ( iplImgPyr->m_ChannelNum != img->nChannels ) )
	{
		// Error Code 5 : Data Size Mismatch
		return( 5 );
	}
	
	// For the Highest Resolution Image
	// Copy the Image Data from the Input Image
	memcpy( iplImgPyr->m_IplImagePyramid[ 0 ]->imageData , img->imageData , ( img->imageSize ) * sizeof( char ) );
	
	// Setting the Image Pyramid Data
	for( int scaleCount = 1 ; scaleCount < iplImgPyr->m_PyramidSize ; ++scaleCount )
	{
		// Obtain the Image at Current Scale
		// By Scaling Down the One from the Next Higher Resolution
		cvResize( iplImgPyr->m_IplImagePyramid[ scaleCount - 1 ] , iplImgPyr->m_IplImagePyramid[ scaleCount ] , CV_INTER_LINEAR );
	}
	
	// Error Code 0 : All Well, That Ends Well
	return( 0 );
}


