/******************************************************************************

   PROJECT:       CVDRIK1
   FILE:          CVDRIKMultiChannelFloatImagePyramid.cpp
   AUTHOR:        PRITHWIJIT GUHA
   DATE:
   DESCRIPTION:   Functions for the Multi-Channel Float Image Pyramid

******************************************************************************/


/*************************** OTHER INCLUDE FILES *****************************/

#include "CVDRIKMultiChannelImage.h"

#define EPSILON 0.0001

/*****************************************************************************/


/******************************************************************************

                                FUNCTIONS

******************************************************************************/

// Function for Initializing the MultiChannelFloatImage Pyramid
// Inputs :-
//			(CVDRIKMultiChannelFloatImagePyramid*) floatImgPyr : Pointer to the MultiChannelFloatImage Pyramid
// Outputs :-
//			None
// Invoked As : initMultiChannelFloatImagePyramid( &floatImgPyr );
void initMultiChannelFloatImagePyramid( CVDRIKMultiChannelFloatImagePyramid* floatImgPyr )
{
	// Memory Allocation Flag
	floatImgPyr->m_MultiChannelFloatImagePyramidMemoryAllocated = false;
	
	// The Image Width at the Highest Resolution
	floatImgPyr->m_HighestResWidth = -1;
	
	// The Image Height at the Highest Resolution
	floatImgPyr->m_HighestResHeight = -1;
	
	// The Number of Channels
	floatImgPyr->m_ChannelNum = -1;
	
	// The Pyramid Size
	floatImgPyr->m_PyramidSize = -1;
	
	// The Pyramid Component Image Array
	floatImgPyr->m_MultiChannelFloatImagePyramid = NULL;
	
	// The Scaling Down Factor Along the Width
	floatImgPyr->m_ScaleFactorWidth = -1.0;
	
	// The Scaling Down Factor Along the Height
	floatImgPyr->m_ScaleFactorHeight = -1.0;
}

// Function for Allocating a MultiChannelFloatImage Pyramid
// Inputs :-
//			(CVDRIKMultiChannelFloatImagePyramid*) floatImgPyr : Pointer to the MultiChannelFloatImage Pyramid
//			(int) highestResWidth : Width of the Highest Resolution Image
//			(int) highestResHeight : Height of the Highest Resolution Image
//			(int) channelNum : The Number of Channels of the Pyramid Component Images
//			(float) scaleFactorWidth : The Down Scaling Factor Along the Width
//			(float) scaleFactorHeight : The Down Scaling Factor Along the Height
//			(int) pyramidSize : The Number of Images in the Pyramid
// Outputs :-
//			(unsigned char) errCode : The Error Code of Execution
// Invoked As : errCode = allocateMultiChannelFloatImagePyramid( floatImgPyr , highestResWidth , highestResHeight , channelNum , scaleFactorWidth , scaleFactorHeight , pyramidSize );
unsigned char allocateMultiChannelFloatImagePyramid( CVDRIKMultiChannelFloatImagePyramid* floatImgPyr , int highestResWidth , int highestResHeight , int channelNum , float scaleFactorWidth , float scaleFactorHeight , int pyramidSize )
{
	// Check Inputs
	if( ( floatImgPyr->m_MultiChannelFloatImagePyramidMemoryAllocated ) || ( highestResWidth <= 0 ) || ( highestResHeight <= 0 ) || ( channelNum <= 0 ) || ( scaleFactorWidth < 1 ) || ( scaleFactorHeight < 1 ) || ( pyramidSize <= 0 ) )
	{
		// Error Code 1 : Invalid Input
		return( 1 );
	}
	
	// Check the Validity of the Sizes
	int finalScaleWidth = cvRound( pow( scaleFactorWidth , ( pyramidSize - 1 ) ) );
	int finalScaleHeight = cvRound( pow( scaleFactorHeight , ( pyramidSize - 1 ) ) );
	if( ( highestResWidth < finalScaleWidth ) || ( highestResHeight < finalScaleHeight ) )
	{
		// Error Code 6 : Index Exceeds Array Bound
		return( 6 );
	}
	
	// Memory Allocation Flag
	floatImgPyr->m_MultiChannelFloatImagePyramidMemoryAllocated = true;
	
	// The Image Width at the Highest Resolution
	floatImgPyr->m_HighestResWidth = highestResWidth;
	
	// The Image Height at the Highest Resolution
	floatImgPyr->m_HighestResHeight = highestResHeight;
	
	// The Number of Channels
	floatImgPyr->m_ChannelNum = channelNum;
	
	// The Pyramid Size
	floatImgPyr->m_PyramidSize = pyramidSize;
	
	// Allocating the Image Pyramid
	floatImgPyr->m_MultiChannelFloatImagePyramid = (CVDRIKMultiChannelFloatImage*) malloc( pyramidSize * sizeof( CVDRIKMultiChannelFloatImage ) );
	
	// Allocating the Indvidual Images
	float currWidth = scaleFactorWidth * highestResWidth;
	float currHeight = scaleFactorHeight * highestResHeight;
	for( int scaleCount = 0 ; scaleCount < pyramidSize ; ++scaleCount )
	{
		// Update the Current Sizes
		currWidth = currWidth / scaleFactorWidth;
		currHeight = currHeight / scaleFactorHeight;
		
		// Allocate the Image at the Current Resolution
		int width = cvRound( currWidth );
		int height = cvRound( currHeight );
		initMultiChannelFloatImage( &floatImgPyr->m_MultiChannelFloatImagePyramid[ scaleCount ] );
		allocateMultiChannelFloatImage( &floatImgPyr->m_MultiChannelFloatImagePyramid[ scaleCount ] , width , height , channelNum );
	}
	
	// The Scaling Down Factor Along the Width
	floatImgPyr->m_ScaleFactorWidth = scaleFactorWidth;
	
	// The Scaling Down Factor Along the Height
	floatImgPyr->m_ScaleFactorHeight = scaleFactorHeight;
	
	// Error Code 0 : All Well, That Ends Well
	return( 0 );
}

// Function for Destroying a MultiChannelFloatImage Pyramid
// Inputs :-
//			(CVDRIKMultiChannelFloatImagePyramid*) floatImgPyr : Pointer to the MultiChannelFloatImage Pyramid
// Outputs :-
//			(unsigned char) errCode : The Error Code of Execution
// Invoked As : errCode = destroyMultiChannelFloatImagePyramid( floatImgPyr );
unsigned char destroyMultiChannelFloatImagePyramid( CVDRIKMultiChannelFloatImagePyramid* floatImgPyr )
{
	// Check Inputs
	if( !( floatImgPyr->m_MultiChannelFloatImagePyramidMemoryAllocated ) )
	{
		// Error Code 1 : Invalid Input
		return( 1 );
	}
	
	// Memory Allocation Flag
	floatImgPyr->m_MultiChannelFloatImagePyramidMemoryAllocated = false;
	
	// The Image Width at the Highest Resolution
	floatImgPyr->m_HighestResWidth = -1;
	
	// The Image Height at the Highest Resolution
	floatImgPyr->m_HighestResHeight = -1;
	
	// The Number of Channels
	floatImgPyr->m_ChannelNum = -1;
	
	// Destroying the Pyramid Component Images
	for( int scaleCount = 0 ; scaleCount < floatImgPyr->m_PyramidSize ; ++scaleCount )
	{
		// Destroying the Multi-Channel Float Image
		destroyMultiChannelFloatImage( &floatImgPyr->m_MultiChannelFloatImagePyramid[ scaleCount ] );
	}
	free( floatImgPyr->m_MultiChannelFloatImagePyramid );
	floatImgPyr->m_MultiChannelFloatImagePyramid = NULL;
	
	// The Pyramid Size
	floatImgPyr->m_PyramidSize = -1;
	
	// The Scaling Down Factor Along the Width
	floatImgPyr->m_ScaleFactorWidth = -1.0;
	
	// The Scaling Down Factor Along the Height
	floatImgPyr->m_ScaleFactorHeight = -1.0;
	
	// Error Code 0 : All Well, That Ends Well
	return( 0 );
}


